@@ -11,9 +11,9 @@ pub(crate) fn horiz_convolution(
 ) {
     let (values, window_size, bounds) = (coeffs.values, coeffs.window_size, coeffs.bounds);
 
-    let normalizer_guard = optimisations::NormalizerGuard::new(values);
+    let normalizer_guard = optimisations::NormalizerGuard16::new(values);
     let precision = normalizer_guard.precision();
-    let coefficients_chunks = normalizer_guard.normalized_i16_chunks(window_size, &bounds);
+    let coefficients_chunks = normalizer_guard.normalized_chunks(window_size, &bounds);
     let initial = 1 << (precision - 1);
 
     let src_rows = src_image.iter_rows(offset);
@@ -28,7 +28,7 @@ pub(crate) fn horiz_convolution(
             for (&k, &src_pixel) in ks.iter().zip(src_pixels) {
                 ss += src_pixel.0 as i32 * (k as i32);
             }
-            dst_pixel.0 = unsafe { optimisations::clip8(ss, precision) };
+            dst_pixel.0 = unsafe { normalizer_guard.clip(ss) };
         }
     }
 }
@@ -41,9 +41,9 @@ pub(crate) fn vert_convolution(
 ) {
     let (values, window_size, bounds) = (coeffs.values, coeffs.window_size, coeffs.bounds);
 
-    let normalizer_guard = optimisations::NormalizerGuard::new(values);
+    let normalizer_guard = optimisations::NormalizerGuard16::new(values);
     let precision = normalizer_guard.precision();
-    let coefficients_chunks = normalizer_guard.normalized_i16_chunks(window_size, &bounds);
+    let coefficients_chunks = normalizer_guard.normalized_chunks(window_size, &bounds);
     let initial = 1 << (precision - 1);
 
     let dst_rows = dst_image.iter_rows_mut();
@@ -58,7 +58,7 @@ pub(crate) fn vert_convolution(
                 let src_pixel = unsafe { src_row.get_unchecked(x_src as usize) };
                 ss += src_pixel.0 as i32 * (k as i32);
             }
-            dst_pixel.0 = unsafe { optimisations::clip8(ss, precision) };
+            dst_pixel.0 = unsafe { normalizer_guard.clip(ss) };
         }
     }
 }
