@@ -1,8 +1,8 @@
 use std::arch::x86_64::*;
 use std::intrinsics::transmute;
 
-use crate::convolution::optimisations::CoefficientsI16Chunk;
-use crate::convolution::{optimisations, Bound, Coefficients};
+use crate::convolution::optimisations::{CoefficientsI16Chunk, NormalizerGuard16};
+use crate::convolution::{optimisations, Coefficients};
 use crate::image_view::{FourRows, FourRowsMut, TypedImageView, TypedImageViewMut};
 use crate::pixels::{Pixel, U8x3};
 use crate::simd_utils;
@@ -17,10 +17,9 @@ pub(crate) fn horiz_convolution(
     let (values, window_size, bounds_per_pixel) =
         (coeffs.values, coeffs.window_size, coeffs.bounds);
 
-    let normalizer_guard = optimisations::NormalizerGuard::new(values);
+    let normalizer_guard = optimisations::NormalizerGuard16::new(values);
     let precision = normalizer_guard.precision();
-    let coefficients_chunks =
-        normalizer_guard.normalized_i16_chunks(window_size, &bounds_per_pixel);
+    let coefficients_chunks = normalizer_guard.normalized_chunks(window_size, &bounds_per_pixel);
     let dst_height = dst_image.height().get();
 
     let src_iter = src_image.iter_4_rows(offset, dst_height + offset);
@@ -51,17 +50,16 @@ pub(crate) fn vert_convolution(
     mut dst_image: TypedImageViewMut<U8x3>,
     coeffs: Coefficients,
 ) {
-    let (values, window_size, bounds) = (coeffs.values, coeffs.window_size, coeffs.bounds);
+    let (values, window_size, bounds_per_pixel) =
+        (coeffs.values, coeffs.window_size, coeffs.bounds);
 
-    let normalizer_guard = optimisations::NormalizerGuard::new(values);
-    let precision = normalizer_guard.precision();
-    let coeffs_i16 = normalizer_guard.normalized_i16();
-    let coeffs_chunks = coeffs_i16.chunks(window_size);
+    let normalizer_guard = optimisations::NormalizerGuard16::new(values);
+    let coefficients_chunks = normalizer_guard.normalized_chunks(window_size, &bounds_per_pixel);
 
     let dst_rows = dst_image.iter_rows_mut();
-    for ((&bound, k), dst_row) in bounds.iter().zip(coeffs_chunks).zip(dst_rows) {
+    for (dst_row, coeffs_chunk) in dst_rows.zip(coefficients_chunks) {
         unsafe {
-            vert_convolution_8u(&src_image, dst_row, k, bound, precision);
+            vert_convolution_8u(&src_image, dst_row, coeffs_chunk, &normalizer_guard);
         }
     }
 }
@@ -410,13 +408,14 @@ unsafe fn horiz_convolution_8u(
 unsafe fn vert_convolution_8u(
     src_img: &TypedImageView<U8x3>,
     dst_row: &mut [U8x3],
-    coeffs: &[i16],
-    bound: Bound,
-    precision: u8,
+    coeffs_chunk: CoefficientsI16Chunk,
+    normalizer_guard: &NormalizerGuard16,
 ) {
     let src_width = src_img.width().get() as usize;
-    let y_start = bound.start;
-    let y_size = bound.size;
+    let y_start = coeffs_chunk.start;
+    let coeffs = coeffs_chunk.values;
+    let max_y = y_start + coeffs.len() as u32;
+    let precision = normalizer_guard.precision();
 
     let initial = _mm_set1_epi32(1 << (precision - 1));
     let initial_256 = _mm256_set1_epi32(1 << (precision - 1));
@@ -433,7 +432,7 @@ unsafe fn vert_convolution_8u(
 
         let mut y: u32 = 0;
 
-        for (s_row1, s_row2) in src_img.iter_2_rows(y_start, y_start + y_size) {
+        for (s_row1, s_row2) in src_img.iter_2_rows(y_start, max_y) {
             // Load two coefficients at once
             let mmk = simd_utils::ptr_i16_to_256set1_epi32(coeffs, y as usize);
 
@@ -455,8 +454,9 @@ unsafe fn vert_convolution_8u(
             y += 2;
         }
 
-        if let Some(s_row) = src_img.get_row(y_start + y) {
-            let mmk = _mm256_set1_epi32(coeffs[y as usize] as i32);
+        if let Some(&k) = coeffs.get(y as usize) {
+            let s_row = src_img.get_row(y_start + y).unwrap();
+            let mmk = _mm256_set1_epi32(k as i32);
 
             let source1 = simd_utils::loadu_si256_raw(s_row, x_in_bytes); // top line
             let source2 = _mm256_setzero_si256(); // bottom line is empty
@@ -499,7 +499,7 @@ unsafe fn vert_convolution_8u(
         let mut sss1 = initial; // right row
         let mut y: u32 = 0;
 
-        for (s_row1, s_row2) in src_img.iter_2_rows(y_start, y_start + y_size) {
+        for (s_row1, s_row2) in src_img.iter_2_rows(y_start, max_y) {
             // Load two coefficients at once
             let mmk = simd_utils::ptr_i16_to_set1_epi32(coeffs, y as usize);
 
@@ -515,8 +515,9 @@ unsafe fn vert_convolution_8u(
             y += 2;
         }
 
-        if let Some(s_row) = src_img.get_row(y_start + y) {
-            let mmk = _mm_set1_epi32(*coeffs.get_unchecked(y as usize) as i32);
+        if let Some(&k) = coeffs.get(y as usize) {
+            let s_row = src_img.get_row(y_start + y).unwrap();
+            let mmk = _mm_set1_epi32(k as i32);
 
             let source1 = simd_utils::loadl_epi64_raw(s_row, x_in_bytes); // top line
             let source2 = _mm_setzero_si128(); // bottom line is empty
@@ -548,7 +549,7 @@ unsafe fn vert_convolution_8u(
     while x_in_bytes < width_in_bytes.saturating_sub(3) {
         let mut sss = initial;
         let mut y: u32 = 0;
-        for (s_row1, s_row2) in src_img.iter_2_rows(y_start, y_start + y_size) {
+        for (s_row1, s_row2) in src_img.iter_2_rows(y_start, max_y) {
             // Load two coefficients at once
             let two_coeffs = simd_utils::ptr_i16_to_set1_epi32(coeffs, y as usize);
 
@@ -562,9 +563,10 @@ unsafe fn vert_convolution_8u(
             y += 2;
         }
 
-        if let Some(s_row) = src_img.get_row(y_start + y) {
+        if let Some(&k) = coeffs.get(y as usize) {
+            let s_row = src_img.get_row(y_start + y).unwrap();
             let pix = simd_utils::mm_cvtepu8_epi32_from_raw(s_row, x_in_bytes);
-            let mmk = _mm_set1_epi32(*coeffs.get_unchecked(y as usize) as i32);
+            let mmk = _mm_set1_epi32(k as i32);
             sss = _mm_add_epi32(sss, _mm_madd_epi16(pix, mmk));
         }
 
@@ -588,14 +590,14 @@ unsafe fn vert_convolution_8u(
 
         for dst_pixel in dst_u8 {
             let mut ss0 = 1 << (precision - 1);
-            for (dy, &k) in coeffs.iter().take(y_size as usize).enumerate() {
+            for (dy, &k) in coeffs.iter().enumerate() {
                 if let Some(src_row) = src_img.get_row(y_start + dy as u32) {
                     let src_ptr = src_row.as_ptr() as *const u8;
                     let src_component = *src_ptr.add(x_in_bytes);
                     ss0 += src_component as i32 * (k as i32);
                 }
             }
-            *dst_pixel = optimisations::clip8(ss0, precision);
+            *dst_pixel = normalizer_guard.clip(ss0);
             x_in_bytes += 1;
         }
     }
