@@ -10,9 +10,9 @@ pub(crate) fn horiz_convolution(
 ) {
     let (values, window_size, bounds) = (coeffs.values, coeffs.window_size, coeffs.bounds);
 
-    let normalizer_guard = optimisations::NormalizerGuard::new(values);
+    let normalizer_guard = optimisations::NormalizerGuard16::new(values);
     let precision = normalizer_guard.precision();
-    let coefficients_chunks = normalizer_guard.normalized_i16_chunks(window_size, &bounds);
+    let coefficients_chunks = normalizer_guard.normalized_chunks(window_size, &bounds);
     let initial = 1 << (precision - 1);
 
     let src_rows = src_image.iter_rows(offset);
@@ -29,8 +29,7 @@ pub(crate) fn horiz_convolution(
                     *s += components[i] as i32 * (k as i32);
                 }
             }
-            dst_pixel.0 =
-                u32::from_le_bytes(ss.map(|v| unsafe { optimisations::clip8(v, precision) }));
+            dst_pixel.0 = u32::from_le_bytes(ss.map(|v| unsafe { normalizer_guard.clip(v) }));
         }
     }
 }
@@ -42,9 +41,9 @@ pub(crate) fn vert_convolution(
 ) {
     let (values, window_size, bounds) = (coeffs.values, coeffs.window_size, coeffs.bounds);
 
-    let normalizer_guard = optimisations::NormalizerGuard::new(values);
+    let normalizer_guard = optimisations::NormalizerGuard16::new(values);
     let precision = normalizer_guard.precision();
-    let coefficients_chunks = normalizer_guard.normalized_i16_chunks(window_size, &bounds);
+    let coefficients_chunks = normalizer_guard.normalized_chunks(window_size, &bounds);
     let initial = 1 << (precision - 1);
 
     let dst_rows = dst_image.iter_rows_mut();
@@ -62,8 +61,7 @@ pub(crate) fn vert_convolution(
                     *s += components[i] as i32 * (k as i32);
                 }
             }
-            dst_pixel.0 =
-                u32::from_le_bytes(ss.map(|v| unsafe { optimisations::clip8(v, precision) }));
+            dst_pixel.0 = u32::from_le_bytes(ss.map(|v| unsafe { normalizer_guard.clip(v) }));
         }
     }
 }
