@@ -2,7 +2,7 @@ use std::num::NonZeroU32;
 use std::slice;
 
 use crate::errors::{CropBoxError, ImageBufferError, ImageRowsError};
-use crate::pixels::{Pixel, PixelType, U8x3, U8x4, F32, I32, U8};
+use crate::pixels::{Pixel, PixelType, U16x3, U8x3, U8x4, F32, I32, U8};
 
 pub(crate) type RowMut<'a, 'b, T> = &'a mut &'b mut [T];
 pub(crate) type TwoRows<'a, T> = (&'a [T], &'a [T]);
@@ -28,6 +28,7 @@ pub struct CropBox {
 pub enum ImageRows<'a> {
     U8x3(Vec<&'a [U8x3]>),
     U8x4(Vec<&'a [U8x4]>),
+    U16x3(Vec<&'a [U16x3]>),
     I32(Vec<&'a [I32]>),
     F32(Vec<&'a [F32]>),
     U8(Vec<&'a [U8]>),
@@ -42,6 +43,7 @@ impl<'a> ImageRows<'a> {
         match self {
             ImageRows::U8x3(rows) => check_rows_count_and_size(width, height, rows),
             ImageRows::U8x4(rows) => check_rows_count_and_size(width, height, rows),
+            ImageRows::U16x3(rows) => check_rows_count_and_size(width, height, rows),
             ImageRows::I32(rows) => check_rows_count_and_size(width, height, rows),
             ImageRows::F32(rows) => check_rows_count_and_size(width, height, rows),
             ImageRows::U8(rows) => check_rows_count_and_size(width, height, rows),
@@ -52,6 +54,7 @@ impl<'a> ImageRows<'a> {
         match self {
             Self::U8x3(_) => PixelType::U8x3,
             Self::U8x4(_) => PixelType::U8x4,
+            Self::U16x3(_) => PixelType::U16x3,
             Self::I32(_) => PixelType::I32,
             Self::F32(_) => PixelType::F32,
             Self::U8(_) => PixelType::U8,
@@ -64,6 +67,7 @@ impl<'a> ImageRows<'a> {
 pub enum ImageRowsMut<'a> {
     U8x3(Vec<&'a mut [U8x3]>),
     U8x4(Vec<&'a mut [U8x4]>),
+    U16x3(Vec<&'a mut [U16x3]>),
     I32(Vec<&'a mut [I32]>),
     F32(Vec<&'a mut [F32]>),
     U8(Vec<&'a mut [U8]>),
@@ -78,6 +82,7 @@ impl<'a> ImageRowsMut<'a> {
         match self {
             Self::U8x3(rows) => check_rows_count_and_size(width, height, rows),
             Self::U8x4(rows) => check_rows_count_and_size(width, height, rows),
+            Self::U16x3(rows) => check_rows_count_and_size(width, height, rows),
             Self::I32(rows) => check_rows_count_and_size(width, height, rows),
             Self::F32(rows) => check_rows_count_and_size(width, height, rows),
             Self::U8(rows) => check_rows_count_and_size(width, height, rows),
@@ -88,6 +93,7 @@ impl<'a> ImageRowsMut<'a> {
         match self {
             Self::U8x3(_) => PixelType::U8x3,
             Self::U8x4(_) => PixelType::U8x4,
+            Self::U16x3(_) => PixelType::U16x3,
             Self::I32(_) => PixelType::I32,
             Self::F32(_) => PixelType::F32,
             Self::U8(_) => PixelType::U8,
@@ -143,6 +149,10 @@ impl<'a> ImageView<'a> {
                 let pixels = align_buffer_to(buffer)?;
                 ImageRows::U8x4(pixels.chunks_exact(width.get() as usize).collect())
             }
+            PixelType::U16x3 => {
+                let pixels = align_buffer_to(buffer)?;
+                ImageRows::U16x3(pixels.chunks_exact(width.get() as usize).collect())
+            }
             PixelType::I32 => {
                 let pixels = align_buffer_to(buffer)?;
                 ImageRows::I32(pixels.chunks_exact(width.get() as usize).collect())
@@ -277,7 +287,7 @@ impl<'a> ImageView<'a> {
         }
     }
 
-    pub(crate) fn u32_image(&self) -> Option<TypedImageView<U8x4>> {
+    pub(crate) fn u8x4_image(&self) -> Option<TypedImageView<U8x4>> {
         if let ImageRows::U8x4(ref rows) = self.rows {
             Some(TypedImageView {
                 width: self.width,
@@ -290,6 +300,19 @@ impl<'a> ImageView<'a> {
         }
     }
 
+    pub(crate) fn u16x3_image(&self) -> Option<TypedImageView<U16x3>> {
+        if let ImageRows::U16x3(ref rows) = self.rows {
+            Some(TypedImageView {
+                width: self.width,
+                height: self.height,
+                crop_box: self.crop_box,
+                rows,
+            })
+        } else {
+            None
+        }
+    }
+
     pub(crate) fn i32_image(&self) -> Option<TypedImageView<I32>> {
         if let ImageRows::I32(ref rows) = self.rows {
             Some(TypedImageView {
@@ -480,6 +503,10 @@ impl<'a> ImageViewMut<'a> {
                 let pixels = align_buffer_to_mut(buffer)?;
                 ImageRowsMut::U8x4(pixels.chunks_exact_mut(width.get() as usize).collect())
             }
+            PixelType::U16x3 => {
+                let pixels = align_buffer_to_mut(buffer)?;
+                ImageRowsMut::U16x3(pixels.chunks_exact_mut(width.get() as usize).collect())
+            }
             PixelType::I32 => {
                 let pixels = align_buffer_to_mut(buffer)?;
                 ImageRowsMut::I32(pixels.chunks_exact_mut(width.get() as usize).collect())
@@ -527,7 +554,7 @@ impl<'a> ImageViewMut<'a> {
         }
     }
 
-    pub(crate) fn u32_image<'s>(&'s mut self) -> Option<TypedImageViewMut<'s, 'a, U8x4>> {
+    pub(crate) fn u8x4_image<'s>(&'s mut self) -> Option<TypedImageViewMut<'s, 'a, U8x4>> {
         if let ImageRowsMut::U8x4(rows) = &mut self.rows {
             Some(TypedImageViewMut {
                 width: self.width,
@@ -539,6 +566,18 @@ impl<'a> ImageViewMut<'a> {
         }
     }
 
+    pub(crate) fn u16x3_image<'s>(&'s mut self) -> Option<TypedImageViewMut<'s, 'a, U16x3>> {
+        if let ImageRowsMut::U16x3(rows) = &mut self.rows {
+            Some(TypedImageViewMut {
+                width: self.width,
+                height: self.height,
+                rows,
+            })
+        } else {
+            None
+        }
+    }
+
     pub(crate) fn i32_image<'s>(&'s mut self) -> Option<TypedImageViewMut<'s, 'a, I32>> {
         if let ImageRowsMut::I32(rows) = &mut self.rows {
             Some(TypedImageViewMut {
